package com.yifu.cloud.plus.v1.yifu.salary.vo;

import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.ExcelAttribute;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.ExcelAttributeConstants;
import com.yifu.cloud.plus.v1.yifu.salary.entity.THaveSalaryNosocial;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;

/**
 * @Author fxj
 * @Date 2022/8/17
 * @Description  有工资无社保导出对象
 * @Version 1.0
 */
@Data
public class THaveSalaryNoSocialExportVo extends THaveSalaryNosocial implements Serializable {

	/**
	 * 客户名称
	 */
	@Length(max = 50, message = "客户名称不能超过50个字符")
	@ExcelAttribute(name = "客户名称", maxLength = 50,needExport=true)
	@Schema(description = "客户名称", name = "customerName")
	private String customerName;
	/**
	 * 结算主体名称
	 */
	@NotBlank(message = "结算主体名称不能为空")
	@Length(max = 50, message = "结算主体名称不能超过50个字符")
	@ExcelAttribute(name = "结算主体名称", isNotEmpty = true, errorInfo = "结算主体名称不能为空", maxLength = 50,needExport=true)
	@Schema(description = "结算主体名称", name = "settlementOrganName")
	private String settlementOrganName;
	/**
	 * 员工名称
	 */
	@Length(max = 32, message = "员工名称不能超过32个字符")
	@ExcelAttribute(name = "员工名称", maxLength = 32,needExport=true)
	@Schema(description = "员工名称", name = "employeeName")
	private String employeeName;
	/**
	 * 身份证号
	 */
	@Length(max = 32, message = "身份证号不能超过32个字符")
	@ExcelAttribute(name = "身份证号", maxLength = 32,needExport=true)
	@Schema(description = "身份证号", name = "employeeIdCard")
	private String employeeIdCard;

	/**
	 * 员工类型
	 */
	@TableField(exist = false)
	@ExcelAttribute(name = "员工类型", errorInfo = "员工类型不能为空", maxLength = 32, isDataId = true, dataType = ExcelAttributeConstants.PERSONNEL_TYPE, needExport = true)
	@Schema(description = "员工类型", name = "empType")
	private String empType;

	/**
	 * 工资月份
	 */
	@NotBlank(message = "工资月份不能为空")
	@ExcelAttribute(name = "工资月份", isNotEmpty = true, errorInfo = "工资月份不能为空",needExport=true)
	@Schema(description = "工资月份", name = "month")
	private String month;

	/**
	 * 应发工资
	 */
	@Schema(description = "应发工资", name = "relaySalary")
	@ExcelAttribute(name = "应发工资", isNotEmpty = true, errorInfo = "应发工资不能为空", needExport = true)
	private String relaySalary;

	/**
	 * 在职状态 （0在职；1离职）
	 */
	@TableField(exist = false)
	@ExcelAttribute(name = "在职状态", errorInfo = "在职状态 0是/1否不能为空", maxLength = 1, isDataId = true, readConverterExp = "0=在职,1=离职", needExport = true)
	@Schema(description = "在职状态 0是/1否", name = "workFlag")
	private String workFlag;

	/**
	 * 原因大类 根据数据字典来判断
	 */
	@TableField(exist = false)
	@Schema(description = "原因大类 根据数据字典来判断 ", name = "reasonType")
	@ExcelAttribute(name = "原因大类", errorInfo = "原因大类不能为空", maxLength = 32, isDataId = true, dataType = ExcelAttributeConstants.HAVE_SALARY_NO_SOCIAL_TYPE, needExport = true)
	private Integer reasonType;

	/**
	 * 创建者-姓名
	 */
	@Schema(description = "创建人-姓名")
	@TableField(fill = FieldFill.INSERT)
	@ExcelProperty("创建人")
	private String createName;
}
