package com.yifu.cloud.plus.v1.yifu.salary.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Maps;
import com.yifu.cloud.plus.v1.yifu.common.core.util.Common;
import com.yifu.cloud.plus.v1.yifu.common.core.util.DateUtil;
import com.yifu.cloud.plus.v1.yifu.common.core.util.ExcelUtil;
import com.yifu.cloud.plus.v1.yifu.common.core.util.R;
import com.yifu.cloud.plus.v1.yifu.common.log.annotation.SysLog;
import com.yifu.cloud.plus.v1.yifu.common.security.annotation.Inner;
import com.yifu.cloud.plus.v1.yifu.salary.entity.SysMessageSalary;
import com.yifu.cloud.plus.v1.yifu.salary.service.SysMessageSalaryService;
import com.yifu.cloud.plus.v1.yifu.salary.vo.SysMessageSalaryHandelExportVo;
import com.yifu.cloud.plus.v1.yifu.salary.vo.SysMessageSalaryTemplExportVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.CharEncoding;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;


/**
 * 最低工资提醒-每月更新一次
 *
 * @author hgw
 * @date 2019-11-21 17:18:10
 */
@RestController
@AllArgsConstructor
@RequestMapping("/sysmessagesalary")
@Tag(name = "最低工资提醒-每月更新一次")
@Slf4j
public class SysMessageSalaryController {

    public static final String EMPLOYEE_ID = "employeeId";
    public static final String MULTIPART_FORM_DATA = "multipart/form-data";
    public static final String CONTENT_DISPOSITION = "Content-Disposition";
    public static final String ATTACHMENT_FILENAME = "attachment;filename=";
    private final SysMessageSalaryService sysMessageSalaryService;
   /* @Autowired
    private RemoteSysDictService remoteSysDictService;
    @Autowired
    private PreRemoteBasicArchivesService preRemoteBasicArchivesService;
    @Autowired
    private RemoteUserService remoteUserService;
    @Autowired
    private RemoteBasicArchivesService remoteBasicArchivesService;*/

    /**
     * 简单分页查询
     *
     * @param page             分页对象
     * @param sysMessageSalary 最低工资提醒-每月更新一次
     * @return
     */
    @Operation(description = "简单分页查询")
    @GetMapping("/page")
    public R<IPage<SysMessageSalary>> getSysMessageSalaryPage(Page<SysMessageSalary> page, SysMessageSalary sysMessageSalary) {
        return new R<>(sysMessageSalaryService.getSysMessageSalaryPage(page, sysMessageSalary));
    }


    /**
     * 简单分页查询
     *
     * @param page             分页对象
     * @param sysMessageSalary 最低工资提醒-每月更新一次
     * @return 查询所有数据，没有数据权限
     */
    @Operation(description = "处理反馈简单分页查询")
    @GetMapping("/handle/page")
    public R<IPage<SysMessageSalary>> getSysMessageSalaryHandlePage(Page<SysMessageSalary> page, SysMessageSalary sysMessageSalary) {
        return new R<>(sysMessageSalaryService.getSysMessageSalaryHandlePage(page, sysMessageSalary));
    }


    /**
     * 通过id查询单条记录
     *
     * @param id
     * @return R
     */
    @Operation(description = "id查询")
    @GetMapping("/{id}")
    public R<SysMessageSalary> getById(@PathVariable("id") String id) {
        return new R<>(sysMessageSalaryService.getById(id));
    }

    /**
     * 生成最低工资提醒
     * hgw
     * 2019-11-21 18:35:56
     */
    @Inner
    @Operation(description = "生成最低工资提醒")
    @GetMapping("/inner/createSysMessageSalary")
    public void createSysMessageSalary() {
        //结算月
        String settleMonth = DateUtil.addMonth(-1);
        //1：删除当前结算月数据：
        sysMessageSalaryService.deleteAllBySettleMonth(settleMonth);
        //2:开始生成：
        //2.1：普通工资
        sysMessageSalaryService.insertSalaryBySettleMonth(settleMonth);

    }

    /**
     * @param
     * @Author: wangan
     * @Date: 2021/1/11
     * @Description: 为了生成之前的月份
     * @return: void
     **/
    @Operation(description = "生成最低工资提醒外部接口调用")
    @SysLog("生成最低工资提醒外部接口调用")
    @PostMapping("/handle/createSysMessageSalary")
    public void handleCreateSysMessageSalary(String settleMonth) {
        //1：删除当前结算月数据：
        sysMessageSalaryService.deleteAllBySettleMonth(settleMonth);
        //2:开始生成：
        //2.1：普通工资
        sysMessageSalaryService.insertSalaryBySettleMonth(settleMonth);

    }


    /**
     * @param
     * @Author: wangan
     * @Date: 2020/12/17
     * @Description: 最低工资提醒-临时人员连续购买最低工资次数
     * @return: void
     **/
    @Inner
    @GetMapping("/inner/createSysMessageSalaryTemp")
    public void createSysMessageSalaryTemp() {
        //结算月
        String settleMonth = DateUtil.addMonth(-1);
        sysMessageSalaryService.createSysMessageSalaryTemp(settleMonth);

    }

    /**
     * @param
     * @Author: wangan
     * @Date: 2021/1/11
     * @Description: 为了生成之前的月份
     * @return: void
     **/
    @Operation(description = "临时人员连续购买最低工资次数外部接口调用")
    @SysLog("临时人员连续购买最低工资次数外部接口调用")
    @PostMapping("/handle/createSysMessageSalaryTemp")
    public void handleCreateSysMessageSalaryTemp(String settleMonth) {
        sysMessageSalaryService.createSysMessageSalaryTemp(settleMonth);

    }


    /**
     * @param
     * @Author: wangan
     * @Date: 2019/11/26
     * @Description: 获取导出的中文字段
     * @return: com.yifu.cloud.v1.common.core.util.R
     **/
    @Operation(description = "获取导出的中文字段")
    @GetMapping("/getExportFieldName")
    public R<List<String>> getExportFieldName() {
        return ExcelUtil.getExportfieldsName(SysMessageSalary.class);
    }


    @GetMapping("/export")
    @Operation(description = "导出")
    @SysLog("导出")
    public void exportHandle(HttpServletResponse response, HttpServletRequest request,
                             @RequestParam(name = "idStr", required = false) String[] idstr,
                             @RequestParam(name = "exportFields", required = true) String[] exportFields, SysMessageSalary vo) {
        List<SysMessageSalary> list;
        OutputStream ouputStream = null;
        HSSFWorkbook workbook;
        ExcelUtil<SysMessageSalary> util1;
        util1 = new ExcelUtil<>(SysMessageSalary.class);
        if (Common.isNotNull(idstr)) {
            list = sysMessageSalaryService.getByIdExport(idstr);
        } else {
            list = sysMessageSalaryService.getExport(vo);
        }
        try {
            HashMap<String, String> dicMap = Maps.newHashMap();
            //获取所有用户
            if (!list.isEmpty()) {
                /*Map<String, String> userNameMap = null;//RemoteExportUtil.getUserNameMap(remoteUserService);
                //字典数据
                R<List<SysDict>> res = remoteSysDictService.findDetailsByItemTypeAndClientId("", ServiceNameConstants.SERVICE_NAME_WXHR, SecurityConstants.FROM_IN);
                //区域数据
                R<List<SysArea>> sysAreaR = preRemoteBasicArchivesService.getSysAreaList(SecurityUtils.getUser());
                dicMap = ServiceUtil.initMapForExport(null == res ? null : res.getData(), null, null == sysAreaR ? null : sysAreaR.getData(), userNameMap);
                //获取员工
                Map<String, TEmployeeInfo> employeeInfoMap = RemoteExportUtil.getTEmployeeInfoMap(remoteBasicArchivesService, Common.listObjectToStr(list, EMPLOYEE_ID, CommonConstants.COMMA_STRING));
                //塞员工值
                for (SysMessageSalary salary : list) {
                    TEmployeeInfo employeeInfo = employeeInfoMap.get(salary.getEmployeeId());
                    if (employeeInfo != null) {
                        salary.setEmpType(employeeInfo.getEmpType());
                        salary.setFileProvince(employeeInfo.getFileProvince());
                        salary.setFileCity(employeeInfo.getFileCity());
                        salary.setFileTown(employeeInfo.getFileTown());
                        salary.setWorkFlag(employeeInfo.getWorkFlag());
                        salary.setWorkingStatusSub(employeeInfo.getWorkingStatusSub());
                    }
                }*/
            }
            workbook = null;// util1.listToExcelByDicMap(list, exportFields, "最低工资提醒信息", dicMap, DateUtil.ISO_EXPANDED_DATE_FORMAT);
            String fileName = "最低工资提醒信息" + LocalDateTime.now() + ".xls";
            response.setContentType(MULTIPART_FORM_DATA);
            response.setHeader(CONTENT_DISPOSITION, ATTACHMENT_FILENAME + URLEncoder.encode(fileName, CharEncoding.UTF_8));
            ouputStream = response.getOutputStream();
            workbook.write(ouputStream);
            ouputStream.flush();
        } catch (Exception e) {
            log.error("执行异常", e);
        } finally {
            try {
                if (null != ouputStream) {
                    ouputStream.close();
                }
            } catch (IOException e) {
                log.error("执行异常", e);
            }
        }
    }


    /**
     * @param
     * @Author: wangan
     * @Date: 2020/12/16
     * @Description:
     * @return: com.yifu.cloud.v1.common.core.util.R<java.util.List < java.lang.String>>
     **/
    @Operation(description = "获取反馈审核导出的中文字段")
    @GetMapping("/getHandleExportFieldName")
    public R<List<String>> getHandleExportFieldName() {
        Class<? super SysMessageSalaryHandelExportVo> superClazz = SysMessageSalaryHandelExportVo.class.getSuperclass();
        R<List<String>> superExportfieldsName = ExcelUtil.getExportfieldsName(superClazz);
        R<List<String>> exportfieldsName = ExcelUtil.getExportfieldsName(SysMessageSalaryHandelExportVo.class);
        List<String> data = superExportfieldsName.getData();
        data.addAll(exportfieldsName.getData());
        return R.ok(data );
    }

    /**
     * @param response
     * @param request
     * @param idstr
     * @param exportFields
     * @param vo
     * @Author: wangan
     * @Date: 2020/12/17
     * @Description:
     * @return: void
     **/
    @GetMapping("/handle/export")
    @Operation(description = "反馈审核人员导出")
    @SysLog("反馈审核人员导出")
    public void exportHandle(HttpServletResponse response, HttpServletRequest request,
                             @RequestParam(name = "idStr", required = false) String[] idstr,
                             @RequestParam(name = "exportFields", required = true) String[] exportFields,
                             SysMessageSalaryHandelExportVo vo) {
        if (Common.isEmpty(vo.getSettleMonth())) {
            return;
        }
        List<SysMessageSalaryHandelExportVo> list;
        OutputStream ouputStream = null;
        HSSFWorkbook workbook;
        ExcelUtil<SysMessageSalaryHandelExportVo> util1;
        util1 = new ExcelUtil<>(SysMessageSalaryHandelExportVo.class);
        if (Common.isNotNull(idstr)) {
            list = sysMessageSalaryService.getByIdHandleExport(idstr);
        } else {
            list = sysMessageSalaryService.getHandleExport(vo);
        }
        try {
            //通过员工id+结算月份查询所有消息提醒数据。
            SimpleDateFormat sf = new SimpleDateFormat("yyyyMM");
            Calendar instance = Calendar.getInstance();
            Date date = sf.parse(vo.getSettleMonth());
            instance.setTime(date);
            instance.add(Calendar.MONTH, -1);
            String lastOneMonth = sf.format(instance.getTime());
            instance.add(Calendar.MONTH, -1);
            String lastTwoMonth = sf.format(instance.getTime());
            instance.add(Calendar.MONTH, -1);
            String lastThreeMonth = sf.format(instance.getTime());
            List<SysMessageSalary> lastTHaveSalaryNosocial = sysMessageSalaryService.getLastTHaveSalaryNosocial(lastOneMonth, lastTwoMonth, lastThreeMonth);
            HashMap<String, Integer> lastHashMap = Maps.newHashMap();
            for (SysMessageSalary entity : lastTHaveSalaryNosocial) {
                lastHashMap.put(entity.getSettleMonth() + entity.getEmployeeId(), null);
            }
            HashMap<String, String> dicMap = Maps.newHashMap();
            //获取所有用户
            if (!list.isEmpty()) {
               /* Map<String, String> userNameMap = null;// RemoteExportUtil.getUserNameMap(remoteUserService);
                //字典数据
                R<List<SysDict>> res = remoteSysDictService.findDetailsByItemTypeAndClientId("", ServiceNameConstants.SERVICE_NAME_WXHR, SecurityConstants.FROM_IN);
                //区域数据
                R<List<SysArea>> sysAreaR = preRemoteBasicArchivesService.getSysAreaList(SecurityUtils.getUser());
                dicMap = ServiceUtil.initMapForExport(null == res ? null : res.getData(), null, null == sysAreaR ? null : sysAreaR.getData(), userNameMap);
                //获取员工
                Map<String, TEmployeeInfo> employeeInfoMap = RemoteExportUtil.getTEmployeeInfoMap(remoteBasicArchivesService, Common.listObjectToStr(list, EMPLOYEE_ID, CommonConstants.COMMA_STRING));
                //塞员工值
                for (SysMessageSalaryHandelExportVo handelExportVo : list) {
                    TEmployeeInfo employeeInfo = employeeInfoMap.get(handelExportVo.getEmployeeId());
                    if (employeeInfo != null) {
                        handelExportVo.setEmpType(employeeInfo.getEmpType());
                        handelExportVo.setFileProvince(employeeInfo.getFileProvince());
                        handelExportVo.setFileCity(employeeInfo.getFileCity());
                        handelExportVo.setFileTown(employeeInfo.getFileTown());
                        handelExportVo.setWorkFlag(employeeInfo.getWorkFlag());
                        handelExportVo.setWorkingStatusSub(employeeInfo.getWorkingStatusSub());
                    }
                    //从map汇总获取前三个月的反馈数据
                    handelExportVo.setLastOneReasonType(lastHashMap.get(lastOneMonth + handelExportVo.getEmployeeId()));
                    handelExportVo.setLastTwoReasonType(lastHashMap.get(lastTwoMonth + handelExportVo.getEmployeeId()));
                    handelExportVo.setLastThreeReasonType(lastHashMap.get(lastThreeMonth + handelExportVo.getEmployeeId()));
                }*/
            }
            workbook = null;//util1.listToExcelByDicMap(list, exportFields, "最低工资反馈导出", dicMap, DateUtil.ISO_EXPANDED_DATE_FORMAT);
            String fileName = "最低工资反馈导出" + LocalDateTime.now() + ".xls";
            response.setContentType(MULTIPART_FORM_DATA);
            response.setHeader(CONTENT_DISPOSITION, ATTACHMENT_FILENAME + URLEncoder.encode(fileName, CharEncoding.UTF_8));
            ouputStream = response.getOutputStream();
            workbook.write(ouputStream);
            ouputStream.flush();
        } catch (Exception e) {
            log.error("执行异常", e);
        } finally {
            try {
                if (null != ouputStream) {
                    ouputStream.close();
                }
            } catch (IOException e) {
                log.error("执行异常", e);
            }
        }
    }


    /**
     * @param
     * @Author: wangan
     * @Date: 2020/12/16
     * @Description:
     * @return: com.yifu.cloud.v1.common.core.util.R<java.util.List < java.lang.String>>
     **/
    @Operation(description = "获取临时人员导出的中文字段")
    @GetMapping("/getTempExportFieldName")
    public R<List<String>> getTempExportFieldName() {
        Class<? super SysMessageSalaryTemplExportVo> superClazz = SysMessageSalaryTemplExportVo.class.getSuperclass();
        R<List<String>> superExportfieldsName = ExcelUtil.getExportfieldsName(superClazz);
        R<List<String>> exportfieldsName = ExcelUtil.getExportfieldsName(SysMessageSalaryTemplExportVo.class);
        List<String> data = superExportfieldsName.getData();
        data.addAll(exportfieldsName.getData());
        return R.ok(data );
    }

    /**
     * @param response
     * @param request
     * @param idstr
     * @param exportFields
     * @param vo
     * @Author: wangan
     * @Date: 2020/12/17
     * @Description:
     * @return: void
     **/
    @GetMapping("/temp/export")
    @Operation(description = "临时人员导出")
    @SysLog("临时人员导出")
    public void tempExportHandle(HttpServletResponse response, HttpServletRequest request,
                                 @RequestParam(name = "idStr", required = false) String[] idstr,
                                 @RequestParam(name = "exportFields", required = true) String[] exportFields,
                                 SysMessageSalaryTemplExportVo vo) {
        if (Common.isEmpty(vo.getSettleMonth())) {
            return;
        }
        List<SysMessageSalaryTemplExportVo> list;
        OutputStream ouputStream = null;
        HSSFWorkbook workbook;
        ExcelUtil<SysMessageSalaryTemplExportVo> util1;
        util1 = new ExcelUtil<>(SysMessageSalaryTemplExportVo.class);
        list = sysMessageSalaryService.getTempExport(vo.getSettleMonth());

        try {
            //通过员工id+结算月份查询所有消息提醒数据。
            SimpleDateFormat sf = new SimpleDateFormat("yyyyMM");
            Calendar instance = Calendar.getInstance();
            Date date = sf.parse(vo.getSettleMonth());
            instance.setTime(date);
            instance.add(Calendar.MONTH, -1);
            String lastOneMonth = sf.format(instance.getTime());
            instance.add(Calendar.MONTH, -1);
            String lastTwoMonth = sf.format(instance.getTime());
            instance.add(Calendar.MONTH, -1);
            String lastThreeMonth = sf.format(instance.getTime());
            List<SysMessageSalary> lastTHaveSalaryNosocial = sysMessageSalaryService.getLastTHaveSalaryNosocial(lastOneMonth, lastTwoMonth, lastThreeMonth);
            HashMap<String, Integer> lastHashMap = Maps.newHashMap();
            for (SysMessageSalary entity : lastTHaveSalaryNosocial) {
                lastHashMap.put(entity.getSalaryMonth() + entity.getEmployeeId(), null);
            }
            HashMap<String, String> dicMap = Maps.newHashMap();
            //获取所有用户
            if (!list.isEmpty()) {
               /* Map<String, String> userNameMap = RemoteExportUtil.getUserNameMap(remoteUserService);
                //字典数据
                R<List<SysDict>> res = remoteSysDictService.findDetailsByItemTypeAndClientId("", ServiceNameConstants.SERVICE_NAME_WXHR, SecurityConstants.FROM_IN);
                //区域数据
                R<List<SysArea>> sysAreaR = preRemoteBasicArchivesService.getSysAreaList(SecurityUtils.getUser());
                dicMap = ServiceUtil.initMapForExport(null == res ? null : res.getData(), null, null == sysAreaR ? null : sysAreaR.getData(), userNameMap);
                //获取员工
                Map<String, TEmployeeInfo> employeeInfoMap = RemoteExportUtil.getTEmployeeInfoMap(remoteBasicArchivesService, Common.listObjectToStr(list, EMPLOYEE_ID, CommonConstants.COMMA_STRING));
                //塞员工值
                for (SysMessageSalaryTemplExportVo exportVo : list) {
                    TEmployeeInfo employeeInfo = employeeInfoMap.get(exportVo.getEmployeeId());
                    if (employeeInfo != null) {
                        *//*exportVo.setEmpType(employeeInfo.getEmpType());
                        exportVo.setFileProvince(employeeInfo.getFileProvince());
                        exportVo.setFileCity(employeeInfo.getFileCity());
                        exportVo.setFileTown(employeeInfo.getFileTown());
                        exportVo.setWorkFlag(employeeInfo.getWorkFlag());
                        exportVo.setWorkingStatusSub(employeeInfo.getWorkingStatusSub());*//*
                    }
                    //从map汇总获取前三个月的反馈数据
                    exportVo.setLastOneReasonType(lastHashMap.get(lastOneMonth + exportVo.getEmployeeId()));
                    exportVo.setLastTwoReasonType(lastHashMap.get(lastTwoMonth + exportVo.getEmployeeId()));
                    exportVo.setLastThreeReasonType(lastHashMap.get(lastThreeMonth + exportVo.getEmployeeId()));
                }*/
            }
            workbook = null;//util1.listToExcelByDicMap(list, exportFields, "最低工资临时人员", dicMap, DateUtil.ISO_EXPANDED_DATE_FORMAT);
            String fileName = "最低工资临时人员" + LocalDateTime.now() + ".xls";
            response.setContentType(MULTIPART_FORM_DATA);
            response.setHeader(CONTENT_DISPOSITION, ATTACHMENT_FILENAME + URLEncoder.encode(fileName, CharEncoding.UTF_8));
            ouputStream = response.getOutputStream();
            workbook.write(ouputStream);
            ouputStream.flush();
        } catch (Exception e) {
            log.error("执行异常", e);
        } finally {
            try {
                if (null != ouputStream) {
                    ouputStream.close();
                }
            } catch (IOException e) {
                log.error("执行异常", e);
            }
        }
    }


    /**
     * @param
     * @Author: wangan
     * @Date: 2019/11/22
     * @Description: 工作台消息提醒：最低工资提醒
     * @return: com.yifu.cloud.v1.common.core.util.R<java.util.Map < java.lang.String, java.lang.Object>>
     **/
    /*@PostMapping("/inner/workBranch/getSalaryMessageWarn")
    @Inner
    public R<List<WorkBanchWarnVo>> getSalaryMessageWarn(@RequestBody List<TSettleDomainSelectVo> remoteSettleDomainVoData) {
        List<WorkBanchWarnVo> resultList = Lists.newArrayList();
        List<String> settleDomainIds = Common.listObjectToStrList(remoteSettleDomainVoData, "id");
        //最低工资提醒
        int lowestalaryWarnCount = 0;
        if (settleDomainIds != null && !settleDomainIds.isEmpty()) {
            lowestalaryWarnCount = sysMessageSalaryService.count(Wrappers.<SysMessageSalary>query().lambda()
                    .in(SysMessageSalary::getDepartId, settleDomainIds));

        }
        if (lowestalaryWarnCount > 0) {
            WorkBanchWarnVo noBuyInsuranceWorkBanchWarnVo = new WorkBanchWarnVo();
            noBuyInsuranceWorkBanchWarnVo.setMesage(lowestalaryWarnCount + "个最低工资提醒");
            noBuyInsuranceWorkBanchWarnVo.setCount(lowestalaryWarnCount);
            noBuyInsuranceWorkBanchWarnVo.setUrl(WorkBanchConstants.LOWERST_SALARY_WARN_NAME_URL);
            resultList.add(noBuyInsuranceWorkBanchWarnVo);
        }
        return new R(resultList);
    }*/


}
